/***************************************************************************

  dynGssEAZF.cpp

  Author: Takashi U. Ito
  e-mail: tuito@post.j-parc.jp

***************************************************************************/

/***************************************************************************
 *   Copyright (C) 2024 by Takashi U. Ito                             *
 *   tuito@post.j-parc.jp                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dynGssEAZF.h"



Double_t dynGssEAZF::gzf_limit(Double_t t, Double_t delta, Double_t Q, Double_t nu) const {
  return ( 1.0/3.0+2.0/3.0*(1.0-(1.0-Q)*delta*delta*t*t)*exp(-0.5*(1.0-Q)*delta*delta*t*t) )*exp(-2.0*Q*delta*delta*t/nu);
}

Double_t dynGssEAZF::gzf(Double_t t, Double_t delta, Double_t Q, Double_t nu) const
{ 

  Double_t a,b,c,suba,subb,subc;
  a=find_n(t,delta); b=find_k(Q); c=find_l(delta,nu);
  suba=a-floor(a); subb=b-floor(b); subc=c-floor(c);
  if(suba==0.0) suba+=0.001; //approximation
  if(subb==0.0) subb+=0.001; //approximation
  if(subc==0.0) subc+=0.001; //approximation
  if(suba==1.0) suba-=0.001; //approximation
  if(subb==1.0) subb-=0.001; //approximation
  if(subc==1.0) subc-=0.001; //approximation

  Double_t p000,p100,p010,p001,p110,p101,p011,p111;
  p000=table[(Int_t)floor(a)][(Int_t)floor(b)][(Int_t)floor(c)];
  p100=table[(Int_t)floor(a)+1][(Int_t)floor(b)][(Int_t)floor(c)];
  p010=table[(Int_t)floor(a)][(Int_t)floor(b)+1][(Int_t)floor(c)];
  p001=table[(Int_t)floor(a)][(Int_t)floor(b)][(Int_t)floor(c)+1];
  p110=table[(Int_t)floor(a)+1][(Int_t)floor(b)+1][(Int_t)floor(c)];
  p101=table[(Int_t)floor(a)+1][(Int_t)floor(b)][(Int_t)floor(c)+1];
  p011=table[(Int_t)floor(a)][(Int_t)floor(b)+1][(Int_t)floor(c)+1];
  p111=table[(Int_t)floor(a)+1][(Int_t)floor(b)+1][(Int_t)floor(c)+1];

  Double_t w000,w100,w010,w001,w110,w101,w011,w111,result;
  w000=1.0/(suba*subb*subc);
  w100=1.0/((1.0-suba)*subb*subc);
  w010=1.0/(suba*(1.0-subb)*subc);
  w001=1.0/(suba*subb*(1.0-subc));
  w110=1.0/((1.0-suba)*(1.0-subb)*subc);
  w101=1.0/((1.0-suba)*subb*(1.0-subc));
  w011=1.0/(suba*(1.0-subb)*(1.0-subc));
  w111=1.0/((1.0-suba)*(1.0-subb)*(1.0-subc));

  result=(w000*p000+w100*p100+w010*p010+w001*p001+w110*p110+w101*p101+w011*p011+w111*p111)/(w000+w100+w010+w001+w110+w101+w011+w111);

  return result;

}

Double_t dynGssEAZF::find_n(Double_t t, Double_t delta) const {
  Int_t i=0;
  while(i<N_DT){
    if((Double_t)i*DT*DELTA0 <= t*delta) i++;
    else break;
  }
  i=i-1;
  return (Double_t)i+(t*delta-(Double_t)i*DT*DELTA0)/(DT*DELTA0);
}

Double_t dynGssEAZF::find_k(Double_t Q) const {
  Int_t i=0;
  while(i<N_Q)
    {
      if(meas_Q[i] <= Q) i++;
      else break;
    }
  i=i-1;
  return (Double_t)i+(Q-meas_Q[i])/(meas_Q[i+1]-meas_Q[i]);
}

Double_t dynGssEAZF::find_l(Double_t delta, Double_t nu) const{
  Int_t i=0;
  while(i<N_NU)
    {
      if(meas_NUonD[i] <= nu/delta) i++;
      else break;
    }
  i=i-1;
  return (Double_t)i+(nu/delta-meas_NUonD[i])/(meas_NUonD[i+1]-meas_NUonD[i]);
}


//#################################################//

ClassImp(dynGssEAZF)  // for the ROOT dictionary

Double_t dynGssEAZF::operator()(Double_t x, const vector<Double_t> &par) const {
  assert(par.size()==3); // make sure the number of parameters handed to the function is correct

  Double_t delta=par[1];
  if(par[1] < 1.0e-5) delta=1.0e-5;

  //return 0 when parameters are out of range
  if((par[0]<0)||(par[0]>1)) return 0.0;
  if((x<0)||(x*delta>=DELTA0*DT*(N_DT-2))) return 0.0;
  if(par[2]<0) return 0.0;

  if((par[2]/delta)>=NARROW_LIM){
      return dynGssEAZF::gzf_limit(x,delta,par[0],par[2]);
  }
  else{
      return dynGssEAZF::gzf(x,delta,par[0],par[2]);
  }

}
